! netcdf_filename.f90 - object-oriented netCDF file interface (filename)
! vi: set sw=4:
! Copyright (C) TOYODA Eizi, 2000.  All rights reserved.
!
! 2000-01-12 ˭ıѻ	
! 2000-01-16 ˭ıѻ	ϥå
! 2000-02-22 ˭ıѻ	Ĺʸ

module netcdf_filename
    use iso_varying_string
    implicit none

    private
    public:: NetcdfSaveFilename, NetcdfIdToFilename
    public:: NetcdfDisableFileId
    public:: NetcdfFilenameId

    character(len = *), parameter:: RCSID = &
	& '$Id: netcdf_filename.f90,v 1.2 2000/03/22 10:38:29 toyoda Exp $'

    integer, parameter:: INITIAL_TABLE_SIZE = 16

    type TableEntry
    	integer::			id
	type(VARYING_STRING)::		name
    end type

    type(TableEntry), save, pointer::	Table(:)

contains

    integer function lookup_id(id)
	integer, intent(in)::		id
	integer::			i
    continue
	do, i = 1, size(Table)
	    if (Table(i)%id == id) then
		lookup_id = i
		return
	    endif
	enddo
	lookup_id = -1
    end function

    function init_table()
	logical::		init_table
	integer::		status
    continue
	init_table = .true.
	if (associated(Table)) return
	allocate(Table(INITIAL_TABLE_SIZE), stat=status)
	if (status == 0) then
	    Table%id = -1
	else
	    init_table = .false.
	endif
    end function

    subroutine growTable(newsize)
	integer, intent(in)::		newsize
	integer::			copysize, status
	type(TableEntry), pointer::	newtable(:)
    continue
	copysize = min(size(Table), newsize)
	allocate(newtable(newsize), stat=status)
	if (status == 0) then
	    newtable(1:copysize) = Table(1:copysize)
	    newtable(copysize+1:)%id = -1
	    deallocate(Table)
	    Table => newtable
	endif
    end subroutine

    integer function NetcdfFilenameId(filename) result(result)
	type(VARYING_STRING), intent(in):: filename
	integer:: i
    continue
	do, i = 1, size(Table)
	    if (Table(i)%id == -1) cycle
	    if (Table(i)%name == filename) then
		result = Table(i)%id; return
	    endif
	enddo
	result = -1
    end function

    subroutine NetcdfDisableFileId(id)
	integer, intent(in)::		id
	integer::			i
    continue
	i = lookup_id(id)
	Table(i)%id = -1
    end subroutine

    type(VARYING_STRING) function NetcdfIdToFilename(id)
	integer, intent(in)::		id
	integer::			i
    continue
	i = lookup_id(id)
	NetcdfIdToFilename = Table(i)%name
    end function

    subroutine NetcdfSaveFilename(id, name)
	integer, intent(in)::		id
	character(len=*), intent(in)::	name
	integer::			i
    continue
	if (.not. init_table()) return
	i = lookup_id(id)
	if (i /= -1) then
	    Table(i)%name = name
	    return
	endif
	i = lookup_id(-1)
	if (i /= -1) then
	    Table(i)%id = id
	    Table(i)%name = name
	    return
	endif
	call growTable(size(Table) * 2)
	i = lookup_id(-1)
	if (i /= -1) then
	    Table(i)%id = id
	    Table(i)%name = name
	endif
    end subroutine

end module netcdf_filename
